@@ -1,4 +1,4 @@
--- | Internal functionality exposed for sibling libraries such as sandwich-webdriver to use. Should not be used otherwise.
+-- | Internal functionality exposed for debugging or for sibling libraries such as sandwich-webdriver to use. Should not be used otherwise.
 
 module Test.Sandwich.Internal (
   Spec
@@ -11,7 +11,7 @@ module Test.Sandwich.Internal (
   , ExampleM
   , ExampleT(..)
 
-  -- For tests
+  -- * For tests
   , RunNodeWithStatus(..)
   , RunNodeFixed
   , RunNode
@@ -26,6 +26,13 @@ module Test.Sandwich.Internal (
   , SomeAsyncExceptionWithEq(..)
   , logEntryStr
 
+  -- * For debugging
+  , getRunTree
+  , getRunTree'
+  , printRunTree
+
+  , commandLineOptions
+
   , module Test.Sandwich.Internal.Formatters
   , module Test.Sandwich.Internal.Running
   ) where
@@ -37,4 +44,5 @@ import Test.Sandwich.Types.RunTree
 import Test.Sandwich.Types.Spec
 
 import Test.Sandwich.Internal.Formatters
+import Test.Sandwich.Internal.Inspection
 import Test.Sandwich.Internal.Running
