@@ -35,7 +35,10 @@ module Test.Sandwich.Misc (
   -- * Context classes
   , BaseContext
   , HasBaseContext
+  , HasBaseContextMonad
   , HasCommandLineOptions
+  , SomeCommandLineOptions(..)
+  , HasSomeCommandLineOptions
 
   -- * Result types
   , Result(..)
