@@ -1,25 +1,28 @@
-{-# LANGUAGE RankNTypes #-}
-{-# LANGUAGE PolyKinds #-}
-{-# LANGUAGE GeneralizedNewtypeDeriving #-}
-{-# LANGUAGE DeriveTraversable #-}
-{-# LANGUAGE DataKinds #-}
-{-# LANGUAGE StandaloneDeriving #-}
-{-# LANGUAGE TypeOperators #-}
-{-# LANGUAGE TemplateHaskell #-}
-{-# LANGUAGE TypeFamilies #-}
-{-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE GADTs #-}
-{-# LANGUAGE UndecidableInstances #-}
-{-# LANGUAGE InstanceSigs #-}
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE DataKinds #-}
+{-# LANGUAGE DeriveTraversable #-}
+{-# LANGUAGE GADTs #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE InstanceSigs #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE PolyKinds #-}
+{-# LANGUAGE RankNTypes #-}
+{-# LANGUAGE StandaloneDeriving #-}
+{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE TypeApplications #-}
+{-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE TypeOperators #-}
+{-# LANGUAGE UndecidableInstances #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
+{-# OPTIONS_GHC -fno-warn-redundant-constraints #-}
 
 -- | The core Spec/SpecCommand types, used to define the test free monad.
 
 module Test.Sandwich.Types.Spec where
 
 import Control.Applicative
-import Control.Exception.Safe
 import Control.Monad.Base
+import Control.Monad.Catch (MonadCatch, MonadMask, MonadThrow)
 import Control.Monad.Free
 import Control.Monad.Free.TH
 import Control.Monad.IO.Unlift
@@ -30,18 +33,19 @@ import Data.Functor.Classes
 import Data.Kind (Type)
 import Data.Maybe
 import Data.String.Interpolate
+import Data.Time
 import GHC.Stack
 import GHC.TypeLits
-import Safe
-
-#ifndef mingw32_HOST_OS
 import Graphics.Vty.Image (Image)
-#endif
+import Safe
+import Type.Reflection
+import UnliftIO.Exception
 
 #if !MIN_VERSION_base(4,13,0)
 import Control.Monad.Fail
 #endif
 
+#define GHC_9_0_X (__GLASGOW_HASKELL__ >= 900 && __GLASGOW_HASKELL__ < 902)
 
 -- * ExampleM monad
 
@@ -70,7 +74,7 @@ instance (MonadBaseControl b m) => MonadBaseControl b (ExampleT context m) where
   liftBaseWith = defaultLiftBaseWith
   restoreM = defaultRestoreM
 
-instance (Monad m, MonadThrow m) => MonadFail (ExampleT context m) where
+instance (MonadIO m) => MonadFail (ExampleT context m) where
   fail :: (HasCallStack) => String -> ExampleT context m a
   fail = throwIO . Reason (Just callStack)
 
@@ -82,6 +86,20 @@ data Result = Success
             | Cancelled
   deriving (Show, Eq)
 
+data ExtraTimingInfo = ExtraTimingInfo {
+  setupFinishTime :: Maybe UTCTime
+  , teardownStartTime :: Maybe UTCTime
+  }
+
+emptyExtraTimingInfo :: ExtraTimingInfo
+emptyExtraTimingInfo = ExtraTimingInfo Nothing Nothing
+
+mkSetupTimingInfo :: UTCTime -> ExtraTimingInfo
+mkSetupTimingInfo setupFinishTime = ExtraTimingInfo (Just setupFinishTime) Nothing
+
+mkTeardownTimingInfo :: UTCTime -> ExtraTimingInfo
+mkTeardownTimingInfo teardownStartTime = ExtraTimingInfo Nothing (Just teardownStartTime)
+
 data ShowEqBox = forall s. (Show s, Eq s) => SEB s
 instance Show ShowEqBox where show (SEB x) = show x
 instance Eq ShowEqBox where (SEB x1) == (SEB x2) = show x1 == show x2
@@ -105,12 +123,9 @@ data FailureReason = Reason { failureCallStack :: Maybe CallStack
                                        , failureAsyncException :: SomeAsyncExceptionWithEq }
                    | ChildrenFailed { failureCallStack :: Maybe CallStack
                                     , failureNumChildren :: Int }
-#ifndef mingw32_HOST_OS
                    | RawImage { failureCallStack :: Maybe CallStack
                               , failureFallback :: String
                               , failureRawImage :: Image }
-#endif
-
   deriving (Show, Typeable, Eq)
 
 instance Exception FailureReason
@@ -133,7 +148,7 @@ instance Eq SomeAsyncExceptionWithEq where
 
 data Label (l :: Symbol) a = Label
 
-data LabelValue (l :: Symbol) a = LabelValue a
+data LabelValue (l :: Symbol) a = LabelValue { unLabelValue :: a }
 
 -- TODO: get rid of overlapping instance
 -- Maybe look at https://kseo.github.io/posts/2017-02-05-avoid-overlapping-instances-with-closed-type-families.html
@@ -146,6 +161,37 @@ instance {-# OVERLAPPING #-} HasLabel (LabelValue l a :> context) l a where
 instance {-# OVERLAPPING #-} HasLabel context l a => HasLabel (intro :> context) l a where
   getLabelValue l (_ :> ctx) = getLabelValue l ctx
 
+getLabelValueMaybe :: forall context l a. (KnownSymbol l, Typeable a, Typeable context) => Label l a -> context -> Maybe a
+getLabelValueMaybe _ ctx = unLabelValue <$> decomposeContext @(LabelValue l a) ctx
+  where
+    -- | https://stackoverflow.com/questions/78224748/extract-a-maybe-from-a-heterogeneous-collection
+    decomposeContext :: forall needle haystack. (
+      Typeable needle, Typeable haystack
+      ) => haystack -> Maybe needle
+    decomposeContext haystack = case typeOf haystack of
+      (eqTypeRep (typeRep @needle) -> Just HRefl) -> Just haystack
+      App (App (eqTypeRep (typeRep @(:>)) -> Just HRefl) a) b -> let
+        part1 :> part2 = haystack
+        in
+        withTypeable a (decomposeContext part1) <|> withTypeable b (decomposeContext part2)
+      _ -> Nothing
+
+-- | https://stackoverflow.com/questions/78224748/extract-a-maybe-from-a-heterogeneous-collection
+-- extractContext :: forall c a b. (Typeable a, Typeable b, Typeable c) => a :> b -> Maybe c
+-- extractContext (a :> b)
+--   -- handle `Int :> a`
+--   | Just x <- cast a = Just x
+--   -- handle `x :> (y :> z)`
+--   --   extract type (of right-hand side) as:  c `op` d
+--   | App (App op c) d <- typeOf b
+--   --   check op ~ (:>)
+--   , Just HRefl <- eqTypeRep op (TypeRep @(:>))
+--   --   extract Typeable c, Typeable d, and recurse
+--     = withTypeable c $ withTypeable d $ extractContext b
+--   -- handle `x :> Int`
+--   | Just x <- cast b = Just x
+--   -- handle remaining cases
+--   | otherwise = Nothing
 
 -- * Free monad language
 
@@ -363,7 +409,7 @@ introduce :: (HasCallStack, Typeable intro) =>
   -- ^ 'Label' under which to introduce the value
   -> ExampleT context m intro
   -- ^ Action to produce the new value (of type 'intro')
-  -> (intro -> ExampleT context m ())
+  -> ((HasCallStack) => intro -> ExampleT context m ())
   -- ^ Action to clean up the new value
   -> SpecFree (LabelValue l intro :> context) m ()
   -- ^ Child spec tree
@@ -380,7 +426,7 @@ introduce' :: (HasCallStack, Typeable intro) =>
   -- ^ 'Label' under which to introduce the value
   -> ExampleT context m intro
   -- ^ Action to produce the new value (of type 'intro')
-  -> (intro -> ExampleT context m ())
+  -> ((HasCallStack) => intro -> ExampleT context m ())
   -- ^ Action to clean up the new value
   -> SpecFree (LabelValue l intro :> context) m ()
   -- ^ Child spec tree
@@ -399,7 +445,7 @@ introduce'' :: (HasCallStack, Typeable intro) =>
   -- ^ 'Label' under which to introduce the value
   -> ExampleT context m intro
   -- ^ Action to produce the new value (of type 'intro')
-  -> (intro -> ExampleT context m ())
+  -> ((HasCallStack) => intro -> ExampleT context m ())
   -- ^ Action to clean up the new value
   -> SpecFree (LabelValue l intro :> context) m ()
   -- ^ Child spec tree
@@ -413,7 +459,11 @@ introduceWith :: (HasCallStack) =>
   -- ^ String label for this node
   -> Label l intro
   -- ^ 'Label' under which to introduce the value
+#if GHC_9_0_X
   -> ((intro -> ExampleT context m [Result]) -> ExampleT context m ())
+#else
+  -> (((HasCallStack) => intro -> ExampleT context m [Result]) -> ExampleT context m ())
+#endif
   -- ^ Callback to receive the new value and the child tree.
   -> SpecFree (LabelValue l intro :> context) m ()
   -- ^ Child spec tree
@@ -428,7 +478,11 @@ introduceWith' :: (HasCallStack) =>
   -- ^ String label for this node
   -> Label l intro
   -- ^ 'Label' under which to introduce the value
+#if GHC_9_0_X
   -> ((intro -> ExampleT context m [Result]) -> ExampleT context m ())
+#else
+  -> (((HasCallStack) => intro -> ExampleT context m [Result]) -> ExampleT context m ())
+#endif
   -- ^ Callback to receive the new value and the child tree.
   -> SpecFree (LabelValue l intro :> context) m ()
   -- ^ Child spec tree
@@ -445,7 +499,11 @@ introduceWith'' :: (HasCallStack) =>
   -- ^ String label for this node
   -> Label l intro
   -- ^ 'Label' under which to introduce the value
+#if GHC_9_0_X
   -> ((intro -> ExampleT context m [Result]) -> ExampleT context m ())
+#else
+  -> (((HasCallStack) => intro -> ExampleT context m [Result]) -> ExampleT context m ())
+#endif
   -- ^ Callback to receive the new value and the child tree.
   -> SpecFree (LabelValue l intro :> context) m ()
   -- ^ Child spec tree
@@ -700,10 +758,14 @@ aroundEach' loc no l f (Free (Introduce'' loci noi li cl alloc clean subspec nex
 
 -- * ----------------------------------------------------------
 
-unwrapContext :: forall m introduce context. (Monad m) => (ExampleT context m [Result] -> ExampleT context m ()) -> ExampleT (introduce :> context) m [Result] -> ExampleT (introduce :> context) m ()
+unwrapContext :: forall m introduce context. (
+  Monad m
+  ) => (ExampleT context m [Result] -> ExampleT context m ())
+    -> ExampleT (introduce :> context) m [Result]
+    -> ExampleT (introduce :> context) m ()
 unwrapContext f (ExampleT action) = do
-  i :> _ <- ask
-  ExampleT $ withReaderT (\(_ :> context) -> context) $ unExampleT $ f $ ExampleT (withReaderT (i :>) action)
+  i' :> _ <- ask
+  ExampleT $ withReaderT (\(_ :> context) -> context) $ unExampleT $ f $ ExampleT (withReaderT (i' :>) action)
 
 
 -- | Convert a spec to a run tree
@@ -713,4 +775,4 @@ alterTopLevelNodeOptions g (Free x) = Free (x { nodeOptions = g (nodeOptions x)
 alterTopLevelNodeOptions _ x@(Pure _) = x
 
 systemVisibilityThreshold :: Int
-systemVisibilityThreshold = 150
+systemVisibilityThreshold = 200
